@@ -22,6 +22,7 @@
 
 #include <grilo.h>
 #include <glib.h>
+#include <locale.h>
 
 #include "config.h"
 
@@ -815,6 +816,8 @@ main (int argc, char *argv[])
 {
   GError *error = NULL;
 
+  setlocale (LC_ALL, "");
+
   context = g_option_context_new ("OPERATION PARAMETERS...");
   g_option_context_add_main_entries (context, entries, NULL);
   g_option_context_add_group (context, grl_init_get_option_group ());
